/*
 * Copyright © 2018 Apple Inc. and the ServiceTalk project authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.servicetalk.http.netty;

import io.servicetalk.buffer.api.BufferAllocator;
import io.servicetalk.http.api.HttpHeadersFactory;
import io.servicetalk.http.api.HttpProtocolVersion;
import io.servicetalk.http.api.HttpResponse;
import io.servicetalk.http.api.HttpResponseFactory;
import io.servicetalk.http.api.HttpResponseStatus;
import io.servicetalk.http.api.StreamingHttpResponses;

import static io.servicetalk.concurrent.internal.FutureUtils.awaitResult;

final class DefaultHttpResponseFactory implements HttpResponseFactory {
    private final HttpHeadersFactory headersFactory;
    private final BufferAllocator allocator;
    private final HttpProtocolVersion version;

    DefaultHttpResponseFactory(final HttpHeadersFactory headersFactory, final BufferAllocator allocator,
                               final HttpProtocolVersion version) {
        this.headersFactory = headersFactory;
        this.allocator = allocator;
        this.version = version;
    }

    @Override
    public HttpResponse newResponse(final HttpResponseStatus status) {
        return awaitResult(StreamingHttpResponses.newResponse(status, version, headersFactory.newHeaders(), allocator,
                headersFactory).toResponse().toFuture());
    }
}
